#include "STC8A8K64D4.H"
#include "ADC.h"
#include <intrins.h>
#include "oled.h"
#include "delay.h"

//存储ADC DMA转换结果，即ADC DMA的目的地址  
//unsigned char xdata adc_samples_buff[ADC_CH_NUM][ADC_DATA_SIZE]; 
/*
	ADC 控制寄存器（ADC_CONTR）， PWM触发ADC控制
	  符号 			地址	B7			B6			B5			B4			B3 B2 B1 B0
	ADC_CONTR		BCH	 ADC_POWER	 ADC_START	 ADC_FLAG	  ADC_EPWMT		ADC_CHS[3:0]
	
	ADC_POWER：ADC电源控制位 
		0：关闭ADC电源 
		1：打开ADC电源。  
	建议进入空闲模式和掉电模式前将ADC电源关闭，以降低功耗
	
	ADC_START：ADC转换启动控制位。写入1后开始ADC转换，转换完成后硬件自动将此位清零 
		0：无影响。即使ADC已经开始转换工作，写0也不会停止A/D转换。 
		1：开始ADC转换，转换完成后硬件自动将此位清零。 
		
	ADC_FLAG：ADC转换结束标志位。当ADC完成一次转换后，硬件会自动将此位置1，
	并向CPU提出中断请求。此标志位必须软件清零。 
	
	ADC_EPWMT：使能PWM实时触发ADC功能。详情请参考PWM章节 
	
	ADC_CHS[3:0]：ADC 模拟通道选择位 
*/

void initADC()
{
	P1M0 = 0x00;	//设置P1.7高阻输入		P1M0第一位置零
	P1M1 = 0xFF;	//设置P1.7高阻输入		P1M1第一位置一
	ADC_CONTR = ADC_POWER | ADC_channel7;//(ADC_POWER)0x80 | (ADC_channel7)0x07
	delay(1);//给 MCU 的 内部ADC 模块电源打开后，需等待约1ms, 等 MCU内部的 ADC 电源稳后再让 ADC工作；  
	P_SW2 = 0x80;	//P_SW2最高位置一 使用扩展寄存器
	
	/*
		 符号 		地址 		B7 		 B6 B5 			B4 B3 B2 B1 B0
		ADCTIM 	   FEA8H 	  CSSETUP  CSHOLD[1:0] 		 SMPDUTY[4:0] 
			
		
	*CSSETUP：ADC通道选择时间控制Tsetup*		这里是0 所以占用1
	CSSETUP   占用ADC工作时钟数 
		0        1（默认值） 
		1        2
	
	CSHOLD[1:0]：ADC通道选择保持时间控制Thold 这里是01,所以占用2
	CSHOLD[1:0]    占用ADC工作时钟数 
		00 					1 
		01 				2（默认值） 
		10 					3 
		11 					4 
		
	SMPDUTY[4:0]：ADC模拟信号采样时间控制Tduty（注意：SMPDUTY一定不能设置小于01010B） 这里是01010 占用11
	SMPDUTY[4:0]    占用ADC工作时钟数 
		00000 				1 
		00001 				2 
		... 				... 
		01010 			11（默认值） 
		... 				... 
		11110 				31 
		11111 				32
	*/
	ADCTIM = CSHOLD2ADCclk | SMPDUTY11ADCclk;	//(CSHOLD2ADCclk)0x20 | (SMPDUTY11ADCclk)0x0a = 0x2a = 0010 1010  //共26个时钟周期
	
	ADCEXCFG = 0x05;//0b00000101转换4次并取平均值 禁止ETR功能				
	P_SW2 = 0;		//关闭扩展寄存器
	
	/*
		 符号	地址 	B7 		B6 		B5 		B4 		B3 	B2 	B1 	B0
		ADCCFG	DEH		-		-	  RESFMT	-          SPEED[3:0]
						0		0		1		0			0011
		
	*/
	//ADC 配置寄存器（ADCCFG）
	//转换结果右对齐。 ADC_RES 保存结果的高 4 位， ADC_RESL 保存结果的低 8 位
	ADCCFG = RESFMT + ADCspeed128SYSclk;	 //转换结果格式控制位  +  设置 ADC工作时钟频率(RESFMT)0x20 + (ADCspeed128SYSclk)0x03 = 0x23 = 0010 0011
	//EADC：A/D转换中断允许位。 
		//0：禁止A/D转换中断 
		//1：允许A/D转换中断 
	EADC = 1;
	_nop_();_nop_();_nop_();_nop_();_nop_();_nop_();
	ADC_CONTR |= ADC_START;		//ADC_CONTR第一位置一,即ADC上电
	AdcFinished = 1;//标志位
}

void ADC_Isr() interrupt 5	//ADC中断
{
	ADC_CONTR &= ~ADC_FLAG;			//清中断标志位 (ADC_FLAG)~0x20 = 1101 1111
	AdcBufferH = ADC_RES;			//保存结果寄存器值。
	AdcBufferL = ADC_RESL;
	ADC_CONTR |= ADC_START;			//重新启动下一次转换。
}
		





